[Unit]
PartOf=<%= app %>-<%= name %>.target

[Service]
User=<%= user %>
WorkingDirectory=<%= engine.root %>
Environment=PORT=%i
<% engine.env.each_pair do |var,env| -%>
Environment="<%= var %>=<%= env %>"
<% end -%>
ExecStart=/bin/bash -lc 'exec <%= process.command %>'
Restart=always
StandardInput=null
StandardOutput=syslog
StandardError=syslog
SyslogIdentifier=%n
KillMode=mixed
TimeoutStopSec=<%= engine.options[:timeout] %>
